import os
from functools import lru_cache

import yaml  # type: ignore
from pydantic_settings import BaseSettings


class DBSettings(BaseSettings):
    pg_user: str
    pg_pass: str
    pg_host: str
    pg_port: int
    pg_db: str

    @property
    def get_db_url(self) -> str:
        return "postgresql+asyncpg://{}:{}@{}:{}/{}".format(
            self.pg_user,
            self.pg_pass,
            self.pg_host,
            self.pg_port,
            self.pg_db,
        )


class RedisSettings(BaseSettings):
    redis_host: str
    redis_port: str


class Settings(BaseSettings):
    db: DBSettings
    redis: RedisSettings


@lru_cache(maxsize=None)
def get_settings():
    with open(os.getenv("CONFIG_PATH", "./config/api_config.yml")) as f:
        config_data: dict = yaml.safe_load(f)

    if os.getenv("INDOCKER"):
        config_data["db"]["host"] = "db"
        config_data["db"]["port"] = 5432

    settings = Settings(
        db=DBSettings(**config_data["db"]),
        redis=RedisSettings(
            **config_data["redis"],
        ),
    )

    return settings
